# coding=utf-8

__all__ = ['ASYNC_ANNOTATION_PROCESSOR',
           'SCHEDULED_ANNOTATION_PROCESSOR']

ASYNC_ANNOTATION_PROCESSOR = '__async_annotation_processor'
SCHEDULED_ANNOTATION_PROCESSOR = '__scheduled_annotation_processor'
